require 'spec_helper'

module Dockerploy
  describe Deploy do
    let(:fixture_path) { 'spec/fixtures/config.yml' }
    let(:config) { Configuration.new(env: 'test') }
    let(:ssh_client) { double(command: double) }
    let(:shell_client) { double(command: double) }
    let(:options) { {} }
    subject { described_class.new(config, options) }

    before do
      stub_const('Dockerploy::Configuration::DEFAULT_CONFIG_FILE', fixture_path)
      allow(SSHClient).to receive(:new).and_return(ssh_client)
      allow(ShellClient).to receive(:new).and_return(shell_client)
    end

    describe '#deploy' do
      it 'destroys a container' do
        expected_command = 'docker rm -f app_8080'
        expect(ssh_client).to receive(:command).with(expected_command)
        subject.deploy
      end

      it 'creates a container on server' do
        expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app -p container.host:1022:22 -p container.host:8080:80   docker/image
        SHELL
        expected_command.chomp!
        expect(ssh_client).to receive(:command).with(expected_command)
        subject.deploy
      end

      context 'with tag' do
        let(:options) { {tag: 'tag'} }

        it 'creates a container on server with tag' do
          expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app -p container.host:1022:22 -p container.host:8080:80   docker/image:tag
          SHELL
          expected_command.chomp!
          expect(ssh_client).to receive(:command).with(expected_command)
          subject.deploy
        end
      end

      context 'an environment file exists' do
        let(:fixture_path) { 'spec/fixtures/config_with_env_file.yml' }

        it 'creates a container with environment variables' do
          expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app \
-p container.host:1022:22 -p container.host:8080:80  \
-e BRANCH='master' -e APP_ENV='test' docker/image
          SHELL
          expected_command.chomp!
          expect(ssh_client).to receive(:command).with(expected_command)
          subject.deploy
        end
      end

      context 'custom variables exist' do
        let(:fixture_path) { 'spec/fixtures/config_with_custom_env.yml' }

        it 'creates a container with custom environment variables' do
          expected_command1 = <<-SHELL
docker run -d --name app_8080 --hostname app \
-p container.host1:1022:22 -p container.host1:8080:80  \
-e BRANCH='master' -e APP_ENV='test' -e ROLE='web_role' docker/image
          SHELL
          expected_command2 = <<-SHELL
docker run -d --name app_8080 --hostname app \
-p container.host2:1022:22 -p container.host2:8080:80  \
-e BRANCH='master' -e APP_ENV='test' -e ROLE='worker_role' docker/image
          SHELL
          expected_command1.chomp!
          expected_command2.chomp!
          expect(ssh_client).to receive(:command).with(expected_command1)
          expect(ssh_client).to receive(:command).with(expected_command2)
          subject.deploy
        end
      end

      context 'volumes exists' do
        let(:fixture_path) { 'spec/fixtures/config_with_volumes.yml' }

        it 'creates a container with volumes' do
          expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app \
-p container.host:1022:22 -p container.host:8080:80 \
-v /opt/app/shared/log:/var/log   \
docker/image
          SHELL
          expected_command.chomp!
          expect(ssh_client).to receive(:command).with(expected_command)
          subject.deploy
        end
      end

      context 'tagging is true' do
        let(:fixture_path) { 'spec/fixtures/config_with_tagging.yml' }

        it 'pushes a tags to the repository' do
          timestamp = Time.now.strftime('%Y%m%dT%H%M')
          expect(shell_client).to receive(:command).with("git tag test-#{timestamp} test-branch").and_return(true)
          expect(shell_client).to receive(:command).with("git push origin test-#{timestamp}").and_return(true)
          subject.deploy
        end
      end

      context 'with hooks' do
        let(:fixture_path) { 'spec/fixtures/config_with_hooks.yml' }

        it 'runs hooks before/after deploying' do
          expect(shell_client).to receive(:command).with('echo "Started deploying"').ordered
          expect(ssh_client).to receive(:command).with('echo "Access to server"').ordered
          expect(ssh_client).to receive(:command).with('docker rm -f app_8080').ordered
          expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app -p container.host:1022:22 -p container.host:8080:80   docker/image
          SHELL
          expected_command.chomp!
          expect(ssh_client).to receive(:command).with(expected_command).ordered
          expect(shell_client).to receive(:command).with('echo "Finished deploying"').ordered
          expect(ssh_client).to receive(:command).with('echo "Access to server again"').ordered
          subject.deploy
        end
      end

      context 'with hooks in each environment' do
        let(:fixture_path) { 'spec/fixtures/config_with_hooks_for_each_env.yml' }

        it 'runs hooks before/after deploying' do
          expect(shell_client).to receive(:command).with('echo "Started deploying on testing"').ordered
          expect(ssh_client).to receive(:command).with('echo "Access to test server"').ordered
          expect(ssh_client).to receive(:command).with('docker rm -f app_8080').ordered
          expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app -p container.host:1022:22 -p container.host:8080:80   docker/image
          SHELL
          expected_command.chomp!
          expect(ssh_client).to receive(:command).with(expected_command).ordered
          expect(shell_client).to receive(:command).with('echo "Finished deploying on testing"').ordered
          expect(ssh_client).to receive(:command).with('echo "Access to test server again"').ordered
          subject.deploy
        end
      end

      context 'with image_tag' do
        let(:fixture_path) { 'spec/fixtures/config_with_image_tag.yml' }

        it 'creates a container with image tag' do
          expected_command = <<-SHELL
docker run -d --name app_8080 --hostname app -p container.host:1022:22 -p container.host:8080:80   docker/image:v1.0.1
          SHELL
          expected_command.chomp!
          expect(ssh_client).to receive(:command).with(expected_command).ordered
          subject.deploy
        end
      end
    end
  end
end
